 /*
  * 
 * <LIC_AMD_STD>
 * Copyright (c) 2004 Advanced Micro Devices, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * The full GNU General Public License is included in this distribution in the
 * file called COPYING
 * </LIC_AMD_STD>
  * 
  * <CTL_AMD_STD>
  * </CTL_AMD_STD>
  * 
  * <DOC_AMD_STD>
  * This file contains routines to write to and read from the I2C bus.
  * </DOC_AMD_STD>
  * 
  */


/* INCLUDE ROUTINES FOR ACCESS.BUS, IF SPECIFIED */
/* This is for SC1200 systems. */

#if GFX_I2C_ACCESS
#include "i2c_acc.c"
#endif

/* INCLUDE ROUTINES FOR CS5530 GPIOs, IF SPECIFIED */
/* This is for GXLV systems that use GPIOs on the CS5530 for I2C. */

#if GFX_I2C_GPIO
#include "i2c_gpio.c"
#endif

/* WRAPPERS IF DYNAMIC SELECTION */
/* Extra layer to call either ACCESS.bus or GPIO routines. */

#if GFX_I2C_DYNAMIC

/*---------------------------------------------------------------------------
 * gfx_i2c_reset
 *---------------------------------------------------------------------------
 */
int gfx_i2c_reset(unsigned char busnum, short adr, char freq)
{
	int status = GFX_STATUS_UNSUPPORTED;
#if GFX_I2C_ACCESS
	if (gfx_i2c_type & GFX_I2C_TYPE_ACCESS)
		status = acc_i2c_reset(busnum, adr, freq);
#endif
#if GFX_I2C_GPIO
	if (gfx_i2c_type & GFX_I2C_TYPE_GPIO)
		status = gpio_i2c_reset(busnum, adr, freq);
#endif
	return(status);
}

/*---------------------------------------------------------------------------
 * gfx_i2c_select_gpio
 *---------------------------------------------------------------------------
 */
int gfx_i2c_select_gpio(int clock, int data)
{
#if GFX_I2C_ACCESS
	if (gfx_i2c_type & GFX_I2C_TYPE_ACCESS)
		acc_i2c_select_gpio(clock, data);
#endif
#if GFX_I2C_GPIO
	if (gfx_i2c_type & GFX_I2C_TYPE_GPIO)
		gpio_i2c_select_gpio(clock, data);
#endif
	return(0);
}

/*---------------------------------------------------------------------------
 * gfx_i2c_write
 *---------------------------------------------------------------------------
 */
int gfx_i2c_write(unsigned char busnum, unsigned char chipadr, unsigned char subadr, 
	unsigned char bytes, unsigned char * data)
{
	int status = -1;
#if GFX_I2C_ACCESS
	if (gfx_i2c_type & GFX_I2C_TYPE_ACCESS)
		status = acc_i2c_write(busnum, chipadr, subadr, bytes, data);
#endif
#if GFX_I2C_GPIO
	if (gfx_i2c_type & GFX_I2C_TYPE_GPIO)
		status = gpio_i2c_write(busnum, chipadr, subadr, bytes, data);
#endif
	return(status);
}

/*---------------------------------------------------------------------------
 * gfx_i2c_read
 *---------------------------------------------------------------------------
 */
int gfx_i2c_read(unsigned char busnum, unsigned char chipadr, unsigned char subadr, 
	unsigned char bytes, unsigned char * data)
{
	int status = -1;
#if GFX_I2C_ACCESS
	if (gfx_i2c_type & GFX_I2C_TYPE_ACCESS)
		status = acc_i2c_read(busnum, chipadr, subadr, bytes, data);
#endif
#if GFX_I2C_GPIO
	if (gfx_i2c_type & GFX_I2C_TYPE_GPIO)
		status = gpio_i2c_read(busnum, chipadr, subadr, bytes, data);
#endif
	return(status);
}

/*---------------------------------------------------------------------------
 * gfx_i2c_init
 *---------------------------------------------------------------------------
 */
int gfx_i2c_init(void)
{
	int status = -1;
#if GFX_I2C_ACCESS
	if (gfx_i2c_type & GFX_I2C_TYPE_ACCESS)
		status = acc_i2c_init();
#endif
#if GFX_I2C_GPIO
	if (gfx_i2c_type & GFX_I2C_TYPE_GPIO)
		status = gpio_i2c_init();
#endif
	return(status);
}

/*---------------------------------------------------------------------------
 * gfx_i2c_cleanup
 *---------------------------------------------------------------------------
 */
void gfx_i2c_cleanup(void)
{
#if GFX_I2C_ACCESS
	if (gfx_i2c_type & GFX_I2C_TYPE_ACCESS)
		acc_i2c_cleanup();
#endif
#if GFX_I2C_GPIO
	if (gfx_i2c_type & GFX_I2C_TYPE_GPIO)
		gpio_i2c_cleanup();
#endif
}

#endif /* GFX_I2C_DYNAMIC */
	
/* END OF FILE */
